/*
* Follow Focus receiver
* v0.2
* 2016-12-16 by Thomas H Winkler
* <thomas.winkler@iggmp.net>
* 
* using RF24 library by J. Coliz <maniacbug@ymail.com>
*/

/*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 version 2 as published by the Free Software Foundation.
*/

/*
* input
* A0 = battery voltage control
* 
* output
* D6 = PWM servo Focus
* D11 = PWM servo FStop
* D10 = PWM servo Zoom
*
* D3 = Switch1
* D4 = Switch2
*
* D5 = connection status
* D12 = record / battery

for RF24 pinning refer to the text file
*/

#include <SPI.h>
#include <RF24.h>
#include <printf.h>

#include <Servo.h>

/****************** PIN Config ***************************/
/*
 * PWM servo output
 */
#define FOCUS 6
#define FSTOP 11
#define ZOOM 10

/* digital input switch to gnd */
#define SW1 3
#define SW2 4

/* digital LED output with 1k resistor to gnd */
#define STATUS 5
#define ALARM 12

/* analog battery voltage control input */
#define PWR A0

/* Hardware configuration: Set up nRF24L01 radio on SPI bus plus pins 7 & 8 */
RF24 radio(7,8);

/* Servo definition */
Servo FocusServo;
Servo FstopServo;
Servo ZoomServo;

/**********************************************************/
/* rf24 identification data definition */
byte addresses[][6] = {"1Node","2Node"};

// data structure
struct DATA {
  bool sw1;
  bool sw2;
  bool sw3;
  int focus;
  int fstop;
  int zoom;
  unsigned long time;
};

unsigned long timeout;
bool blinkStatus;    

unsigned long lowTime;
bool lowStatus;    

bool online;
float power;

/**********************************************************/
/* setup routine                                          */
void setup() {
  
//  Serial.begin(115200);

/**************************/
/* init radio             */
  radio.begin();

//TODO make channel selectable
  radio.setChannel(108);
  radio.setPALevel(RF24_PA_LOW);
  radio.setRetries(15, 15);
  
  radio.openWritingPipe(addresses[0]);
  radio.openReadingPipe(1,addresses[1]);

  
  // Start the radio listening for data
  radio.startListening();

/**************************/
/* init servo             */
  FocusServo.attach(FOCUS);
  FstopServo.attach(FSTOP);
  ZoomServo.attach(ZOOM);

  // init IO
  pinMode(SW1, OUTPUT);
  pinMode(SW2, OUTPUT);

  pinMode(ALARM, OUTPUT);
  pinMode(PWR, INPUT);

  pinMode(STATUS, OUTPUT);

  digitalWrite(STATUS, LOW);
  digitalWrite(ALARM, LOW);

  online = false;
}


/**********************************************************/
/* main loop                                              */
void loop() {

  int focusVal;
  int fstopVal;
  int zoomVal;
  
  DATA getData;
  unsigned long currentTime;

  /* get radio data */
  if(radio.available()){

    radio.read( &getData, sizeof(DATA) );
    timeout = micros();

    digitalWrite(STATUS, HIGH);
    online = true;


    /* set run switches */
    digitalWrite(SW1, getData.sw1);

    /* red LED on when running
     *  blinking when running
     */

    digitalWrite(ALARM, getData.sw1);


    /* use red LED for low power
    /* send power low to sender */
    power = analogRead(PWR) / 83.7;
  /*    Serial.print(power);
      Serial.println(" Volt");*/
  
    if (power < 7.5 && !getData.sw1) {

      if (micros()-lowTime > 500000) {

        lowTime = micros();
  
        if (lowStatus == LOW)
          lowStatus = HIGH;
        else
          lowStatus = LOW;
  
        digitalWrite(ALARM, HIGH);

      
      /* send low power to sender */
        radio.stopListening();
        radio.write( &power, sizeof(float) );
        radio.startListening();
      }
    }

    else
      digitalWrite(ALARM, getData.sw1);
  

/*    digitalWrite(SW2, getData.sw2);*/

    if (online) {

      /* set servo positions */
      focusVal = map(getData.focus,0,1023,510,2300);
      FocusServo.writeMicroseconds(focusVal);

      fstopVal = map(getData.fstop,0,1023,510,2300);
      FstopServo.writeMicroseconds(fstopVal);

      zoomVal = map(getData.zoom,0,1023,510,2300);
      ZoomServo.writeMicroseconds(zoomVal);
    }

      /* send back timestamp for correct transmisison */
/*    radio.stopListening();
    radio.write( &online, sizeof(bool) );
    radio.startListening();*/
}

  /* if no radio, check for timeout */
  else {

    /* blink LED */
    if (micros()-timeout > 500000) {
      timeout = micros();

      if (blinkStatus == LOW)
        blinkStatus = HIGH;
      else
        blinkStatus = LOW;

      digitalWrite(STATUS, blinkStatus);

      /* stop camera on transmit lost */
//      digitalWrite(SW1, LOW);
//      digitalWrite(ALARM, LOW);
    }
  }
}

